#
# Copyright (C) 1999 Ken MacLeod
# XML::Handler::XMLWriter is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
# $Id: xmlObjectStyle.pm,v 1.1 2003/09/17 14:33:27 bboett Exp $
#

use strict;

package POE::Helper::Leve::xmlObjectStyle;

use UNIVERSAL;

use vars qw{ $VERSION };

# will be substituted by make-rel script
$VERSION = "0.07";

sub new 
{
    my $type = shift;
    my $this = ($#_ == 0) ? { %{ (shift) } } : { @_ };

    return bless $this, $type;
}

sub start_document {
    my ($this, $document) = @_;

    $this->{Names} = [];
    $this->{Nodes} = [];
}

sub end_document {
    my ($this, $document) = @_;

    delete $this->{Names};
    delete $this->{Nodes};

    return();
}

sub start_element {
    my ($this, $element) = @_;

    push @{$this->{Names}}, $element->{Name};
    push @{$this->{Nodes}}, $element;

    my $el_name = "s_" . $element->{Name};
    $el_name =~ s/[^a-zA-Z0-9_]/_/g;
    if ($this->can($el_name)) {
	$this->$el_name($element);
	return 1;
    }

    return 0;
}

sub end_element {
    my ($this, $element) = @_;

    my $called_sub = 0;
    my $el_name = "e_" . $element->{Name};
    $el_name =~ s/[^a-zA-Z0-9_]/_/g;
    if ($this->can(${el_name})) {
	$this->$el_name($element);
	$called_sub = 1;
    }

    pop @{$this->{Names}};
    pop @{$this->{Nodes}};

    return $called_sub;
}

sub in_element {
    my ($this, $name) = @_;

    return ($this->{Names}[-1] eq $name);
}

sub within_element {
    my ($this, $name) = @_;

    my $count = 0;
    foreach my $el_name (@{$this->{Names}}) {
	$count ++ if ($el_name eq $name);
    }

    return $count;
}

1;

__END__

=head1 NAME

POE::Helper::Leve::xmlObjectStyle - a PerlSAX handler base class for calling user-defined subs

=head1 SYNOPSIS

 use POE::Helper::Leve::xmlObjectStyle;

 package MyHandlers;
 use vars qw{ @ISA };

 sub s_NAME { my ($this, $element) = @_ };
 sub e_NAME { my ($this, $element) = @_ };

 $this->{Names};    # an array of names
 $this->{Nodes};    # an array of $element nodes

 $handler = MyHandlers->new();
 $this->in_element($name);
 $this->within_element($name);

=head1 DESCRIPTION

C<POE::Helper::Leve::xmlObjectStyle> is a base class for PerlSAX handlers.
C<POE::Helper::Leve::xmlObjectStyle> is subclassed to implement complete behavior and
to add element-specific handling.

Each time an element starts, a method by that name prefixed with `s_'
is called with the element to be processed.  Each time an element
ends, a method with that name prefixed with `e_' is called.  Any
special characters in the element name are replaced by underscores.

Subclassing POE::Helper::Leve::xmlObjectStyle in this way is similar to
XML::Parser's Subs style.

POE::Helper::Leve::xmlObjectStyle maintains a stack of element names,
`C<$this->{Names}', and a stack of element nodes, `C<$this->{Nodes}>'
that can be used by subclasses.  The current element is pushed on the
stacks before calling an element-name start method and popped off the
stacks after calling the element-name end method.  The
`C<in_element()>' and `C<within_element()>' calls use these stacks.

If the subclass implements `C<start_document()>', `C<end_document()>',
`C<start_element()>', and `C<end_element()>', be sure to use
`C<SUPER::>' to call the the superclass methods also.  See perlobj(1)
for details on SUPER::.  `C<SUPER::start_element()>' and
`C<SUPER::end_element()>' return 1 if an element-name method is
called, they return 0 if no method was called.

POE::Helper::Leve::xmlObjectStyle does not implement any other PerlSAX handlers.

POE::Helper::Leve::xmlObjectStyle supports the following methods:

=over 4

=item new( I<OPTIONS> )

A basic `C<new()>' method.  `C<new()>' takes a list of key, value
pairs or a hash and creates and returns a hash with those options; the
hash is blessed into the subclass.

=item in_element($name)

Returns true if `C<$name>' is equal to the name of the innermost
currently opened element.

=item within_element($name)

Returns the number of times the `C<$name>' appears in Names.

=back

=head1 AUTHOR

Ken MacLeod, ken@bitsko.slc.ut.us

=head1 SEE ALSO

perl(1), PerlSAX.pod(3)

=cut
